
package com.wily.apm.model.webservices.client.interfaces;


import java.net.ConnectException;
import java.net.MalformedURLException;
import java.net.UnknownHostException;

import com.wily.apm.model.webservices.IAgentService;
import com.wily.apm.model.webservices.IAlertsService;
import com.wily.apm.model.webservices.IApmConfigService;
import com.wily.apm.model.webservices.IBusSvcService;
import com.wily.apm.model.webservices.IBusTransService;
import com.wily.apm.model.webservices.IEdgeService;
import com.wily.apm.model.webservices.IMetricsService;
import com.wily.apm.model.webservices.IOwnerService;
import com.wily.apm.model.webservices.ITransactionService;
import com.wily.apm.model.webservices.IVertexService;
import com.wily.apm.model.webservices.IIncidentsService;

/**
 * This interface represents a connection to a given Introscope Enterprise
 * Manager.
 *  
 * It provides the following functionalities:
 * 	 -Information on the configuration of the Introscope 
 *     Enterprise Manager.
 * 	 
 *   -Access to all the Polling Web Services that are available on the 
 *     Introscope Enterprise Manager.
 * 
 * @author sensu01
 *
 */
public interface IApmClient
{

    // Context Path for the APM Web Services
	public static String CONTEXTPATH_AGENT_POLLINGWS = "apm-web-services/services/AgentService?wsdl";
	public static String CONTEXTPATH_ALERTS_POLLINGWS = "apm-web-services/services/AlertsService?wsdl";
    public static String CONTEXTPATH_APM_CONFIG_POLLINGWS = "apm-web-services/services/ApmConfigService?wsdl";
    public static String CONTEXTPATH_APM_VERTEX_POLLINGWS = "apm-web-services/services/VertexService?wsdl";
    public static String CONTEXTPATH_APM_METRICS_POLLINGWS = "apm-web-services/services/MetricsService?wsdl";
    public static String CONTEXTPATH_APM_EDGE_POLLINGWS = "apm-web-services/services/EdgeService?wsdl";
    public static String CONTEXTPATH_APM_OWNER_POLLINGWS = "apm-web-services/services/OwnerService?wsdl";
    public static String CONTEXTPATH_APM_BUSSVC_POLLINGWS = "apm-web-services/services/BusSvcService?wsdl";
    public static String CONTEXTPATH_APM_BUSTRANS_POLLINGWS = "apm-web-services/services/BusTransService?wsdl";
    public static String CONTEXTPATH_APM_TRANS_POLLINGWS = "apm-web-services/services/TransactionService?wsdl";
    public static String CONTEXTPATH_APM_INCIDENTS_POLLINGWS = "apm-web-services/services/IncidentsService?wsdl";
    
	/**
	 * Checks if the EM pointed to by this IIntroscopeClient
	 * 	1. Is running
	 *  2. Can be connected to using the credentials given  
	 *  3. Has the required web services installed
	 * 
	 * @throws MalformedURLException 
	 * @throws UnknownHostException
	 * @throws ConnectException
	 * @throws UserNotAuthorizedException
	 * @throws WebServicesNotInstalledException
	 * 
	 */
	public boolean validate() throws Exception ;
	
	   
    /**
     * Obtains the Agent Polling Web Service 
     * 
     * @return IAgentService
     * @throws Exception
     */
    public IAgentService getAgentServicePollingWS() throws Exception ;
   
    
    /**
     * Obtains the Alerts Polling Web Service 
     * 
     * @return IAlertsService
     * @throws Exception
     */
    public IAlertsService getAlertsServicePollingWS() throws Exception ;

    /**
     * Obtains the Apm Config Polling Web Service 
     * 
     * @return IAlertPollingService
     * @throws Exception
     */
    public IApmConfigService getApmConfigServicePollingWS() throws Exception ;

		
	/**
	 * Obtains the Alerts Polling Web Service 
	 * 
	 * @return IAlertPollingService
	 * @throws Exception
	 */
	public IVertexService getVertexServicePollingWS() throws Exception ;


	/**
	 * Obtains the Metrics Data Web Service 
     *
	 * @return IMetricsDataService
	 * @throws Exception
	 */
	public IEdgeService getEdgeServicePollingWS() throws Exception ;

	/**
	 * Obtains the Metrics List Web Service 
     *
	 * @return IMetricsDataService
	 * @throws Exception
	 */
	public IOwnerService getOwnerServicePollingWS() throws Exception ;
	
	/**
	 * @return
	 * @throws Exception
	 */
	public IMetricsService getMetricsServicePollingWS() throws Exception ;
    
	/**
	 * @return
	 * @throws Exception
	 */
	public IBusSvcService getBusSvcServicePollingWS() throws Exception ;
	
	/**
	 * @return
	 * @throws Exception
	 */
	public IBusTransService getBusTransServicePollingWS() throws Exception ;
	
	/**
	 * @return
	 * @throws Exception
	 */
	public ITransactionService getTransactionServicePollingWS() throws Exception ;

	
	/**
     * @return
     * @throws Exception
     */
    public IIncidentsService getIncidentsServicePollingWS() throws Exception ;
}