
package com.wily.apm.model.webservices.client.manager;

import java.net.ConnectException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.UnknownHostException;

import javax.xml.rpc.ServiceException;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import com.wily.apm.model.webservices.AgentServiceLocator;
import com.wily.apm.model.webservices.AlertsServiceLocator;
import com.wily.apm.model.webservices.ApmConfigServiceLocator;
import com.wily.apm.model.webservices.BusSvcServiceLocator;
import com.wily.apm.model.webservices.BusTransServiceLocator;
import com.wily.apm.model.webservices.EdgeServiceLocator;
import com.wily.apm.model.webservices.IAgentService;
import com.wily.apm.model.webservices.IAlertsService;
import com.wily.apm.model.webservices.IApmConfigService;
import com.wily.apm.model.webservices.IBusSvcService;
import com.wily.apm.model.webservices.IBusTransService;
import com.wily.apm.model.webservices.IEdgeService;
import com.wily.apm.model.webservices.IMetricsService;
import com.wily.apm.model.webservices.IOwnerService;
import com.wily.apm.model.webservices.ITransactionService;
import com.wily.apm.model.webservices.IVertexService;
import com.wily.apm.model.webservices.MetricsServiceLocator;
import com.wily.apm.model.webservices.OwnerServiceLocator;
import com.wily.apm.model.webservices.TransactionServiceLocator;
import com.wily.apm.model.webservices.VertexServiceLocator;
import com.wily.apm.model.webservices.IncidentsServiceLocator;
import com.wily.apm.model.webservices.IIncidentsService;
import com.wily.apm.model.webservices.client.interfaces.IApmClient;
import com.wily.apm.model.webservices.client.interfaces.UserNotAuthorizedException;
import com.wily.apm.model.webservices.client.interfaces.WebServicesNotInstalledException;

/**
 *
 * This is an implementation of the IAPMClient interface. 
 * It represents a connection to a given Introscope Enterprise
 * Manager.
 * 
 * It provides the following functionalities:
 * 	 -Information on the configuration of the Wily APM Introscope 
 *    Enterprise Manager.
 *   -Parameters 
 * 	 -Access to all the Polling Web Services that are available 
 *    on the Wily APM Introscope Enterprise Manager.
 *
 * @author sensu01
 *
 */
public class ApmClient implements IApmClient
{

	private String 	iscopeWebServiceHost; 
	private int 	iscopeWebServicePort;
	private String 	iscopeUser; 
	private String 	iscopePassword; 
	private String  iscopeWebServerProtocol;
	
	private IAgentService     agentService = null;
	private IAlertsService     alertService = null;
    private IApmConfigService apmConfigPollingService = null;
	private IMetricsService   metricsPollingService = null;
	private IBusSvcService    busSvcPollingService = null;
	private IBusTransService  busTransPollingService = null;
	private IEdgeService      edgePollingService = null;
	private IVertexService    vertexPollingService = null;
	private IOwnerService     ownerPollingService = null;
	private ITransactionService transPollingService = null;
    private static Logger       logger              = null;
    private IIncidentsService incidentsPollingService = null;
    private static String trustStoreProperty = "javax.net.ssl.trustStore";
    private static String trustStorePropertyPswd ="javax.net.ssl.trustStorePassword";

    // Sets up log4j logging for this class
    private void initLogger()
    {
        if (logger == null)
        {
            logger = Logger.getLogger(getClass());
            logger.setLevel(Level.DEBUG);
            BasicConfigurator.configure();
        }
    }
    
   
    /**
     * Initializes the CA Wily certificate required for
     * SSL validation
     */
    private void initCertificates()
    {
       /* if (iscopeWebServerProtocol.equalsIgnoreCase("https"))
        {
            File dir1 = new File(".");
            String path = null;
            try 
            {
               path = dir1.getAbsolutePath() + "/resources/cacerts";
            } catch (Exception e) {
                logger.error("Unable to read SSL certificates: " + e.getLocalizedMessage());
                
            }
            logger.info("The path for SSL certificates is: " + path);
             
            System.setProperty(trustStoreProperty, path);
            System.setProperty(trustStorePropertyPswd , "changeit");
           // System.setProperty("javax.net.debug", "all");
        }*/

    }
       
       
	/**
	 * Constructor for constructing a connection to the 
	 * Introscope Enterprise Manager providing the 
	 * input parameters below. 
	 * 
	 * 
	 * @param iscopeWebServiceHost
	 * @param iscopeWebServicePort
	 * @param iscopeUser
	 * @param iscopePassword
	 * @param iscopeWebServerProtocol
	 * @throws Exception 
	 */
	public ApmClient(String iscopeWebServiceHost, int iscopeWebServicePort,
		String iscopeUser, String iscopePassword, String iscopeWebServerProtocol) throws Exception
	{
		
		this.iscopeWebServiceHost = iscopeWebServiceHost;
		this.iscopeWebServicePort = iscopeWebServicePort;
		this.iscopeUser = iscopeUser;
		this.iscopePassword = iscopePassword;
		this.iscopeWebServerProtocol = iscopeWebServerProtocol;
		initLogger();
		initCertificates();
	}
	
	/**
	 * Checks if the EM pointed to by this IIntroscopeClient
	 * 	1. Is running
	 *  2. Can be connected to using the credentials given  
	 *  3. Has the required web services installed
	 * 
	 * @throws MalformedURLException 
	 * @throws WebServicesNotInstalledException
	 * @throws UnknownHostException 
	 * @throws UserNotAuthorizedException 
	 * @throws ConnectException 
	 * 
	 */
	public boolean validate() throws Exception

	{
	    boolean emRunning = false;
		ApmConfigServiceLocator locator = new ApmConfigServiceLocator();		
		IApmConfigService service = locator.getApmConfigService(new URL(getApmConfigServicePollingWSDL()));
		((org.apache.axis.client.Stub) service).setUsername(iscopeUser);
		((org.apache.axis.client.Stub) service).setPassword(iscopePassword);
		emRunning = service.isEMRunning();
		
        return emRunning;	

	}
	
	/**
	 * Obtains the host where the Introscope Web Services will be 
	 * available.
	 * 
	 * @return String
	 */
	public String getIscopeWebServiceHost() {
		return iscopeWebServiceHost;
	}

	/**
	 * Obtains the port where the Introscope Web Services will be 
	 * available.
	 * 
	 * @return int
	 */
	public int getIscopeWebServicePort() {
		return iscopeWebServicePort;
	}	
		

	
	/**
	 * Obtains the password for the userid that is used to login
	 * to the Introscope Enterprise Manager.
     *
	 * @return String
	 */
	public String getIscopePassword() {
		return iscopePassword;
	}
	
	/**
	 * Obtains the userid that is used to login
	 * to the Introscope Enterprise Manager.
     *
	 * @return String
	 */
	public String getIscopeUser() {
		return iscopeUser;
	}

	
	/**
	 * Converts the Host and port to a string
	 *
	 * @return String
	 */
	public String toString(){
		return this.iscopeWebServiceHost + ":" + this.iscopeWebServicePort;
	}

	
	/**
	 * Two IntroscopeClient objects are equal if the iscope web service host and port are the same.
	 */
	public boolean equals(Object inObj) {
		try {
			ApmClient inClient = (ApmClient)inObj;
			if (inClient.getIscopeWebServiceHost().equals(this.getIscopeWebServiceHost()) &&
				inClient.getIscopeWebServicePort() == this.getIscopeWebServicePort()) {
				return true;
			}
		}
		catch (Exception e) {
			
		}
		return false;
	}
	
	public int hashCode() {
		String hashKey = this.getIscopeWebServiceHost() + ":" + this.getIscopeWebServicePort();
		return hashKey.hashCode();
	}

	public String getAlertsServicePollingWSDL() throws Exception
	{
	    String endpointURL = iscopeWebServerProtocol + "://" + iscopeWebServiceHost + ":" + iscopeWebServicePort 
	        + "/" + IApmClient.CONTEXTPATH_ALERTS_POLLINGWS ;
	    return endpointURL;
	}

	public String getAgentServicePollingWSDL() throws Exception
	{
	    String endpointURL = iscopeWebServerProtocol + "://" + iscopeWebServiceHost + ":" + iscopeWebServicePort 
	        + "/" + IApmClient.CONTEXTPATH_AGENT_POLLINGWS ;
	    return endpointURL;
	}
       
    public String getApmConfigServicePollingWSDL() throws Exception
    {
       String endpointURL = iscopeWebServerProtocol + "://" + iscopeWebServiceHost + ":" + iscopeWebServicePort 
        + "/" + IApmClient.CONTEXTPATH_APM_CONFIG_POLLINGWS ;
       return endpointURL;
    }

   
    public String getBusSvcServicePollingWSDL() throws Exception
    {
        String endpointURL = iscopeWebServerProtocol + "://" + iscopeWebServiceHost + ":" + iscopeWebServicePort 
        + "/" + IApmClient.CONTEXTPATH_APM_BUSSVC_POLLINGWS ;
        return endpointURL;
    }

   
    public String getBusTransServicePollingWSDL() throws Exception
    {
        String endpointURL = iscopeWebServerProtocol + "://" + iscopeWebServiceHost + ":" + iscopeWebServicePort 
        + "/" + IApmClient.CONTEXTPATH_APM_BUSTRANS_POLLINGWS ;
        return endpointURL;
    }

   
    public String getEdgeServicePollingWSDL() throws Exception
    {
        String endpointURL = iscopeWebServerProtocol + "://" + iscopeWebServiceHost + ":" + iscopeWebServicePort 
        + "/" + IApmClient.CONTEXTPATH_APM_EDGE_POLLINGWS ;
        return endpointURL;
    }

  
    public String getMetricsServicePollingWSDL() throws Exception
    {
        String endpointURL = iscopeWebServerProtocol + "://" + iscopeWebServiceHost + ":" + iscopeWebServicePort 
        + "/" + IApmClient.CONTEXTPATH_APM_METRICS_POLLINGWS ;
        return endpointURL;
    }

   
    public String getOwnerServicePollingWSDL() throws Exception
    {
        String endpointURL = iscopeWebServerProtocol + "://" + iscopeWebServiceHost + ":" + iscopeWebServicePort 
        + "/" + IApmClient.CONTEXTPATH_APM_OWNER_POLLINGWS;
        return endpointURL;
    }

   
    public String getTransactionServicePollingWSDL() throws Exception
    {
        String endpointURL = iscopeWebServerProtocol + "://" + iscopeWebServiceHost + ":" + iscopeWebServicePort 
        + "/" + IApmClient.CONTEXTPATH_APM_TRANS_POLLINGWS ;
        return endpointURL;
    }

  
    public String getVertexServicePollingWSDL() throws Exception
    {
        String endpointURL = iscopeWebServerProtocol + "://" + iscopeWebServiceHost + ":" + iscopeWebServicePort 
        + "/" + IApmClient.CONTEXTPATH_APM_VERTEX_POLLINGWS ;
        return endpointURL;
    }

 

    public String getIncidentsServicePollingWSDL()
    {
        String endpointURL = iscopeWebServerProtocol + "://" + iscopeWebServiceHost + ":" + iscopeWebServicePort 
        + "/" + IApmClient.CONTEXTPATH_APM_INCIDENTS_POLLINGWS ;
        return endpointURL;
    }
    
   public IAgentService getAgentServicePollingWS() throws Exception
    {
        if (agentService == null) {
            AgentServiceLocator locator = new AgentServiceLocator();        
            agentService = locator.getAgentService(new URL(getAgentServicePollingWSDL()));
            ((org.apache.axis.client.Stub) agentService).setUsername(iscopeUser);
            ((org.apache.axis.client.Stub) agentService).setPassword(iscopePassword);
        }
        
        return agentService;
    }

  
    public IBusSvcService getBusSvcServicePollingWS() throws Exception
    {
        if (busSvcPollingService == null) {
            BusSvcServiceLocator locator = new BusSvcServiceLocator();        
            busSvcPollingService = locator.getBusSvcService(new URL(getBusSvcServicePollingWSDL()));
            ((org.apache.axis.client.Stub) busSvcPollingService).setUsername(iscopeUser);
            ((org.apache.axis.client.Stub) busSvcPollingService).setPassword(iscopePassword);
        }
        
        return busSvcPollingService;
    }

   
    public IBusTransService getBusTransServicePollingWS() throws Exception
    {
        if (busTransPollingService == null) {
            BusTransServiceLocator locator = new BusTransServiceLocator();        
            busTransPollingService = locator.getBusTransService(new URL(getBusTransServicePollingWSDL()));
            ((org.apache.axis.client.Stub) busTransPollingService).setUsername(iscopeUser);
            ((org.apache.axis.client.Stub) busTransPollingService).setPassword(iscopePassword);
        }
        
        return busTransPollingService;
    }

   
    public IEdgeService getEdgeServicePollingWS() throws Exception
    {
        if (edgePollingService == null) {
            EdgeServiceLocator locator = new EdgeServiceLocator();        
            edgePollingService = locator.getEdgeService(new URL(getEdgeServicePollingWSDL()));
            ((org.apache.axis.client.Stub) edgePollingService).setUsername(iscopeUser);
            ((org.apache.axis.client.Stub) edgePollingService).setPassword(iscopePassword);
        }
        
        return edgePollingService;
    }

    public IMetricsService getMetricsServicePollingWS() throws Exception
    {
        if (metricsPollingService == null) {
            MetricsServiceLocator locator = new MetricsServiceLocator();        
            metricsPollingService = locator.getMetricsService(new URL(getMetricsServicePollingWSDL()));
            ((org.apache.axis.client.Stub) metricsPollingService).setUsername(iscopeUser);
            ((org.apache.axis.client.Stub) metricsPollingService).setPassword(iscopePassword);
        }
        
        return metricsPollingService;
    }

   
    public IOwnerService getOwnerServicePollingWS() throws Exception
    {
        if (ownerPollingService == null) {
            OwnerServiceLocator locator = new OwnerServiceLocator();        
            ownerPollingService = locator.getOwnerService(new URL(getOwnerServicePollingWSDL()));
            ((org.apache.axis.client.Stub) ownerPollingService).setUsername(iscopeUser);
            ((org.apache.axis.client.Stub) ownerPollingService).setPassword(iscopePassword);
        }
        
        return ownerPollingService;
    }

   
    public ITransactionService getTransactionServicePollingWS() throws Exception
    {
        if (transPollingService == null) {
            TransactionServiceLocator locator = new TransactionServiceLocator();        
            transPollingService = locator.getTransactionService(new URL(getTransactionServicePollingWSDL()));
            ((org.apache.axis.client.Stub) transPollingService).setUsername(iscopeUser);
            ((org.apache.axis.client.Stub) transPollingService).setPassword(iscopePassword);
        }
        
        return transPollingService;
    }

   
   
    public IVertexService getVertexServicePollingWS() throws Exception
    {
        if (vertexPollingService == null) {
            VertexServiceLocator locator = new VertexServiceLocator();        
            vertexPollingService = locator.getVertexService(new URL(getVertexServicePollingWSDL()));
            ((org.apache.axis.client.Stub) vertexPollingService).setUsername(iscopeUser);
            ((org.apache.axis.client.Stub) vertexPollingService).setPassword(iscopePassword);
        }
        
        return vertexPollingService;
    }

    
    public IApmConfigService getApmConfigServicePollingWS() throws Exception
    {
        if (apmConfigPollingService == null) {
            ApmConfigServiceLocator locator = new ApmConfigServiceLocator();        
            apmConfigPollingService = locator.getApmConfigService(new URL(getApmConfigServicePollingWSDL()));
            ((org.apache.axis.client.Stub) apmConfigPollingService).setUsername(iscopeUser);
            ((org.apache.axis.client.Stub) apmConfigPollingService).setPassword(iscopePassword);
        }
        
        return apmConfigPollingService;
    }


 @Override
    public IIncidentsService getIncidentsServicePollingWS() throws ServiceException, MalformedURLException
    {
        if (incidentsPollingService == null) {
            IncidentsServiceLocator locator = new IncidentsServiceLocator();        
            incidentsPollingService = locator.getIncidentsService(new URL(getIncidentsServicePollingWSDL()));
            ((org.apache.axis.client.Stub) incidentsPollingService).setUsername(iscopeUser);
            ((org.apache.axis.client.Stub) incidentsPollingService).setPassword(iscopePassword);
        }
        
        return incidentsPollingService;
    }

	public IAlertsService getAlertsServicePollingWS() throws Exception
    {
        if (alertService == null) {
            AlertsServiceLocator locator = new AlertsServiceLocator();        
            alertService = locator.getAlertsService(new URL(getAlertsServicePollingWSDL()));
            ((org.apache.axis.client.Stub) alertService).setUsername(iscopeUser);
            ((org.apache.axis.client.Stub) alertService).setPassword(iscopePassword);
        }
        
        return alertService;
    }


}

