/* ---------------------------------------------------------------------- */
/*               Proprietary and Confidential Information                 */
/*                                                                        */
/* This material contains, and is part of a computer software program     */
/* which is, proprietary and confidential information owned by  		  */
/* CA, INC.  The program, including this material, 						  */
/* may not be duplicated, disclosed or reproduced in whole or in part for */
/* any purpose without the express written authorization of CA INC.       */
/* All authorized reproductions must be marked with this      			  */
/* legend.                                                                */
/*                                                                        */
/*                             Copyright 2010                             */
/*                   				CA, Inc.             				  */
/*                           All rights reserved                          */
/*                                                                        */
/* ---------------------------------------------------------------------- */


package com.wily.apm.model.webservices.common;

import java.util.Date;

/**
 * External representation of CEM incident for use in web services etc.
 * 
 * @author macdu01
 *
 */
public class DIncidentSnapshot {
	private long id;
	private Date startDate;
	private Date triggerDate;
	private Date lastModified;
	private Date closeDate;
	private String name;
	private String cause;
	private short defectType;
	private String defectName;
	private String severity;
	private long transactionId;
	private int status;
	private long numberOfDefects;
	private String assignee;

	/**
	 * Set incident id.
	 * @param id
	 *            the incident id to set.
	 */
	public void setId(long id) {
		this.id = id;
	}

	/**
	 * Get the incident id
	 * @return the incident id
	 */
	public long getId() {
		return id;
	}

	/**
	 * Set the date & time at which incident started.
	 * @param startDate 
	 *            the date & time at which incident started.
	 */
	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}

	/**
	 * Get the date & time at which incident started.
	 * @return the date & time at which incident started.
	 */
	public Date getStartDate() {
		return startDate;
	}

	/**
	 * Set the date & time at which incident was triggered.
	 * @param triggerDate
	 *            the date & time at which incident was triggered.
	 */
	public void setTriggerDate(Date triggerDate) {
		this.triggerDate = triggerDate;
	}

	/**
	 * Get the date & time at which incident was triggered.
	 * @return the date & time at which incident was triggered.
	 */
	public Date getTriggerDate() {
		return triggerDate;
	}

	/**
	 * Set the date & time at which incident was last modified.
	 * @param lastModified
	 *            the date & time at which incident was last modified.
	 */
	public void setLastModified(Date lastModified) {
		this.lastModified = lastModified;
	}

	/**
	 * Get the date & time at which incident was last modified.
	 * @return the date & time at which incident was last modified.
	 */
	public Date getLastModified() {
		return lastModified;
	}

	/**
	 * Set the date & time at which incident was closed.
	 * @param closeDate
	 *            the date & time at which incident was closed.
	 */
	public void setCloseDate(Date closeDate) {
		this.closeDate = closeDate;
	}

	/**
	 * Get the date & time at which incident was closed.
	 * @return the date & time at which incident was closed.
	 */
	public Date getCloseDate() {
		return closeDate;
	}

	/**
	 * Set the incident name
	 * @param name
	 *            the incident name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Get the incident name.
	 * @return the incident name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Set the cause of the incident.
	 * @param cause
	 *            the cause of the incident to set.
	 */
	public void setCause(String cause) {
		this.cause = cause;
	}

    /**
	 * Get the cause of the incident.
	 * @return the cause of the incident.
	 */
	public String getCause() {
		return cause;
	}

    /**
     * Set the defect type of the incident.
     * @param defectType
     *            the defectType of the incident to set.
     */
    public void setDefectType(short defectType)
    {
        this.defectType = defectType;
    }


    /**
     * Get the defect type of the incident.
     * @return the defect type of the incident.
     */
    public short getDefectType()
    {
        return defectType;
    }

	/**
	 * Set name of defect.
	 * @param defectName
	 *            the defect name to set.
	 */
	public void setDefectName(String defectName) {
		this.defectName = defectName;
	}

	/**
	 * Get name of defect.
	 * @return the defect name.
	 */
	public String getDefectName() {
		return defectName;
	}

	/**
	 * Set incident severity.
	 * @param severity
	 *            the severity to set.
	 */
	public void setSeverity(String severity) {
		this.severity = severity;
	}

	/**
	 * Get incident severity.
	 * @return the severity.
	 */
	public String getSeverity() {
		return severity;
	}

	/**
	 * Set ID of business transaction that incident is related to. 
	 * @param transactionId
	 *            the ID of business transaction that incident is related to.
	 */
	public void setTransactionId(long transactionId) {
		this.transactionId = transactionId;
	}

	/**
	 * SGet ID of business transaction that incident is related to. 
	 * @return the ID of business transaction that incident is related to.
	 */
	public long getTransactionId() {
		return transactionId;
	}

	/**
	 * Set the incident status.
	 * @param status
	 *            the status to set.
	 */
	public void setStatus(int status) {
		this.status = status;
	}

	/**
	 * Get the incident status.
	 * @return the status.
	 */
	public int getStatus() {
		return status;
	}

	/**
	 * Set the number of defects associated with this incident.
	 * @param numberOfDefects
	 *            the number of defects associated with this incident.
	 */
	public void setNumberOfDefects(long numberOfDefects) {
		this.numberOfDefects = numberOfDefects;
	}

	/**
	 * Get the number of defects associated with this incident.
	 * @return thenumber of defects associated with this incident.
	 */
	public long getNumberOfDefects() {
		return numberOfDefects;
	}

	/**
	 * Set name of person assigned to this incident.
	 * @param assignee
	 *            the name of person assigned to this incident.
	 */
	public void setAssignee(String assignee) {
		this.assignee = assignee;
	}

	/**
	 * Get name of person assigned to this incident. 
	 * @return the name of person assigned to this incident.
	 */
	public String getAssignee() {
		return assignee;
	}
}
